package com.ruoyi.web.controller.common;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.syb.util.SybUtil;
import com.ruoyi.common.pay.tm.TmPayService;
import com.ruoyi.common.pay.tm.enums.PayStatus;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.EtCallbackLog;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.mapper.EtCallbackLogMapper;
import com.ruoyi.system.service.CallbackService;
import com.wechat.pay.java.core.exception.ValidationException;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.TreeMap;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 支付回调
 */
@Slf4j
@RestController
@RequestMapping("/payment/callback")
public class CallbackController {

    @Autowired
    private CallbackService callbackService;

    @Autowired
    private TmPayService tmPayService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Resource
    private EtCallbackLogMapper callbackLogMapper;

    /**
     * 微信支付回调
     */
    @ApiOperation(value = "微信支付回调")
    @RequestMapping(value = "/wechat", method = RequestMethod.POST)
    public ResponseEntity<Boolean> weChat(HttpServletRequest request) {
        try {
            synchronized (callbackService) { // 注意这里的锁对象选择很重要
                callbackService.weChat(request);
            }
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

    /**
     * 微信退款回调
     */
    @ApiOperation(value = "微信退款回调")
    @RequestMapping(value = "/wechat/refund", method = RequestMethod.POST)
    public ResponseEntity<Boolean> weChatRefund(HttpServletRequest request) {
        try {
            callbackService.weChatRefund(request);
        } catch (ValidationException e) {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(null);
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
        }
        return ResponseEntity.status(HttpStatus.OK).body(null);
    }

    /**
     * 太米微信支付回调
     */
    @ApiOperation(value = "太米微信支付回调")
    @RequestMapping(value = "/tmwx", method = RequestMethod.POST)
    public String tmwx(HttpServletRequest request) {
        try {
            request.setCharacterEncoding("UTF-8");//通知传输的编码为GBK
            // 获取所有参数
            TreeMap<String, String> params = SybUtil.getParams(request);
            log.info("【太米微信支付回调】接收对象 : " + JSON.toJSONString(params));
            // 验证签名
            boolean sign = tmPayService.validSign(params);
            if (sign) {
                EtCallbackLog etCallbackLog = new EtCallbackLog();
                etCallbackLog.setBody(JSON.toJSONString(params));
                etCallbackLog.setType("1");
                String payType = params.get("payType"); // 交易类型
                String outTradeNo = params.get("outTradeId");   // 商户自定义订单号
                String trxStatus = params.get("payStatus");    // 交易结果
                String remark = params.get("remark");    // 附加信息，用于判断业务类型
                AttachVo attachVo = JSONObject.parseObject(remark,AttachVo.class);
                log.info("【太米微信支付回调】回调参数--附加信息 : 【{}】",JSON.toJSONString(attachVo));
                if(PayStatus.isSuccess(trxStatus) && payType.equals("wx_pay")) {
                    // 新版支付订单
                    callbackService.businessHandle(outTradeNo,attachVo, ServiceConstants.PAY_TYPE_TLWX);
                }
                //异步保存回调日志
                asynchronousSaveCallbackLog(etCallbackLog);
            }
            return "error";
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    private void asynchronousSaveCallbackLog(EtCallbackLog etCallbackLog) {
        //开异步线程保存回调参数
        scheduledExecutorService.schedule(() -> {
            log.info("【微信支付回调】异步保存回调参数");
            etCallbackLog.setCreateTime(DateUtils.getNowDate());
            int i = callbackLogMapper.insertEtCallbackLog(etCallbackLog);
            if(i>0){
                log.info("【微信支付回调】异步保存回调参数成功");
            }
        }, 0, TimeUnit.SECONDS);
    }


}
