package com.ruoyi.common.pay.tm;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.tm.vo.RefundInfo;
import com.ruoyi.common.pay.tm.vo.TmTradeInfo;
import com.ruoyi.common.pay.wx.Payable;
import com.ruoyi.common.pay.wx.RefundAble;
import com.ruoyi.common.utils.http.HttpUtils;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import org.springframework.stereotype.Service;
import java.util.HashMap;
import java.util.TreeMap;

/**
 * 太米支付
 */
@Service
public class TmPayService {

    private final static String SIGNKEY = "ac6d97e67b444b7a43edfc9182634786";

    /**
     * 订单查询
     */
    public static TmTradeInfo orderQuery(IChannelInfo channel, String outTradeNo) {
        HashMap<String, String> body = new HashMap<String, String>();
        body.put("outTradeId", outTradeNo);
        body.put("terminalType", "1");
        body.put("shopId", channel.getShopId());
        String response = doPost("/open/Pay/orderQuery", body,channel);
        if (com.ruoyi.common.utils.StringUtils.hasText(response)) {
            JSONObject jsonResponse = JSON.parseObject(response);
            if (jsonResponse.getInteger("errCode") == 0 && "ok".equals(jsonResponse.getString("errMsg"))) {
                JSONObject tradeInfo = jsonResponse.getJSONObject("tradeInfo");
                TmTradeInfo tmTradeInfo = tradeInfo.toJavaObject(TmTradeInfo.class);
                return tmTradeInfo;
            } else {
                throw new ServiceException("订单查询失败: " + jsonResponse.getString("errMsg"));
            }
        }
        throw new ServiceException("订单查询数据为空");
    }

    /**
     * 退款
     */
    public static RefundInfo refund(IChannelInfo channel,RefundAble refundAble) {
        HashMap<String, String> body = new HashMap<String, String>();
        body.put("refundFee", String.valueOf(refundAble.getAmount()));
        body.put("terminalType", "1");
        body.put("outTradeId", refundAble.getOutRefundNo());
        body.put("shopId", channel.getShopId());
        String response = doPost("/open/Pay/refund", body,channel);
        if (com.ruoyi.common.utils.StringUtils.hasText(response)) {
            JSONObject jsonResponse = JSON.parseObject(response);
            if (jsonResponse.getInteger("errCode") == 0 && "ok".equals(jsonResponse.getString("errMsg"))) {
                JSONObject tradeInfo = jsonResponse.getJSONObject("tradeInfo");
                RefundInfo refundInfo = tradeInfo.toJavaObject(RefundInfo.class);
                return refundInfo;
            } else {
                throw new ServiceException("退款失败: " + jsonResponse.getString("errMsg"));
            }
        }
        throw new ServiceException("退款数据为空");
    }

    /**
     * 关闭订单
     */
    public static void closeOrder(IChannelInfo channel, String outTradeNo) {
        HashMap<String, String> body = new HashMap<String, String>();
        body.put("outTradeId", outTradeNo);
        body.put("terminalType", "1");
        body.put("shopId", channel.getShopId());
        doPost("/open/Pay/orderClose", body,channel);
    }

    /**
     * 小程序支付
     */
    public static PrepayWithRequestPaymentResponse pay(IChannelInfo channel, Payable payable) {
        HashMap<String, String> body = new HashMap<>();
        body.put("payAmount", String.valueOf(payable.getAmount()));
        body.put("terminalType", "1");
        body.put("shopId", channel.getShopId());  // 从渠道获取shopId
        body.put("sn", channel.getSn());
        body.put("payType", "wx_pay");
        body.put("outTradeId", payable.getOutTradeNo());
        body.put("body", payable.getDescription());
        body.put("notifyUrl", channel.getNotifyUrl());
        String response = doPost(channel.getHttpUrl() + "/open/Pay/miniPay", body, channel);

        if (com.ruoyi.common.utils.StringUtils.hasText(response)) {
            // 解析 response，提取 "params" 字段
            JSONObject jsonResponse = JSON.parseObject(response);
            if (jsonResponse.getInteger("errCode") == 0 && "ok".equals(jsonResponse.getString("errMsg"))) {
                // 提取 "params" 并转成 PrepayWithRequestPaymentResponse 对象
                JSONObject params = jsonResponse.getJSONObject("params");
                PrepayWithRequestPaymentResponse paymentResponse = params.toJavaObject(PrepayWithRequestPaymentResponse.class);
                return paymentResponse;
            } else {
                throw new ServiceException("支付失败: " + jsonResponse.getString("errMsg"));
            }
        }
        throw new ServiceException("支付数据为空");
    }



    private static String doPost(String url, HashMap<String, String> body, IChannelInfo channel) {
        body.put("developerId", channel.getDeveloperId());
        body.put("version", "1.0");
        body.put("timestamp", String.valueOf(System.currentTimeMillis() / 1000));
        body.put("nonceStr", StringUtils.getRandomString(16));

        String bodyStr = StringUtils.getAsciiSort(body);
        String sign = Md5Utils.getMD5Code(bodyStr + "&key=" + channel.getSignKey()).toUpperCase();
        body.put("sign", sign);

        HashMap<String, String> headerData = new HashMap<>();
        headerData.put("Content-Type", "application/json");

        String response = HttpUtils.sendPostWithHeaders(url, headerData, JSON.toJSONString(body));
        return response;
    }

    public boolean validSign(TreeMap<String, String> params) {
        // 获取传递过来的签名
        String receivedSign = params.get("sign");  // 获取签名字段
        if (receivedSign == null) {
            return false;  // 如果没有传递签名，验签失败
        }
        // 移除签名字段后，重新生成签名
        params.remove("sign");
        // 按照请求时的签名逻辑，生成签名字符串
        String paramsStr = StringUtils.getAsciiSort(params);  // 按ASCII排序
        String generatedSign = Md5Utils.getMD5Code(paramsStr + "&key=" + SIGNKEY).toUpperCase();  // 重新生成签名
        // 比较签名是否一致
        return generatedSign.equals(receivedSign);
    }

}
